@@ -21,18 +21,10 @@ __author__ = "Xavier Alvarez Delgado (xalvarez@merabytes.com)"
 __coauthor__ = "Juan Ramón Higueras Pica (juanramon.higueras@wsg127.com)"
 
 parser = argparse.ArgumentParser()
-parser.add_argument("-d", "--download",
-                    dest="download_input",
-                    help="Download file contents remotely from the acido blob.",
-                    action='store')
 parser.add_argument("-f", "--fleet",
                     dest="fleet",
                     help="Create new fleet.",
                     action='store')
-parser.add_argument("-x", "--launch",
-                    dest="scan",
-                    help="Launch a new scan.",
-                    action='store')
 parser.add_argument("-n", "--num-instances",
                     dest="num_instances",
                     help="Instances that the operation affect",
@@ -45,10 +37,6 @@ parser.add_argument("-e", "--exec",
                     dest="exec_cmd",
                     help="Execute command in all selected instances.",
                     action='store')
-parser.add_argument("-sh", "--shell",
-                    dest="shell",
-                    help="Execute command and upload to blob.",
-                    action='store')
 parser.add_argument("-s", "--select",
                     dest="select",
                     help="Select instances matching name/regex.",
@@ -61,14 +49,26 @@ parser.add_argument("-c", "--config",
                     dest="config",
                     help="Start configuration of acido.",
                     action='store_true')
-parser.add_argument("-cp", "--copy",
-                    dest="copy",
-                    help="Copy remote file.",
+parser.add_argument("-w", "--wait",
+                    dest="wait",
+                    help="Number of seconds to wait for the command to finish.",
                     action='store_true')
-parser.add_argument("-ps", "--interactive",
+parser.add_argument("-i", "--input-file",
+                    dest="input_file",
+                    help="The name of the file to split.",
+                    action='store_true')
+parser.add_argument("-in", "--interactive",
                     dest="interactive",
                     help="Start interactive acido session.",
                     action='store_true')
+parser.add_argument("-sh", "--shell",
+                    dest="shell",
+                    help="Execute command and upload to blob.",
+                    action='store')
+parser.add_argument("-d", "--download",
+                    dest="download_input",
+                    help="Download file contents remotely from the acido blob.",
+                    action='store')
 
 
 args = parser.parse_args()
@@ -78,13 +78,14 @@ instances_outputs = {}
 
 def build_output(result):
     global instances_outputs
-    instances_outputs[result[0]] = result[1]
+    instances_outputs[result[0]] = [result[1], result[2]]
+    print(instances_outputs)
 
-def exec_command(rg, cg, cont, command, max_retries):
+def exec_command(rg, cg, cont, command, max_retries, input_file):
     env = os.environ.copy()
     env["PATH"] = "/usr/sbin:/sbin:" + env["PATH"]
     # Kill tmux window
-    subprocess.Popen(["tmux", "kill-session", "-t", cont], env=env)
+    subprocess.Popen(["tmux", "kill-session", "-t", cont], env=env, stdout=subprocess.DEVNULL)
     time.sleep(2)
     subprocess.Popen(["tmux", "new-session", "-d", "-s", cont], env=env)
     time.sleep(5)
@@ -92,6 +93,9 @@ def exec_command(rg, cg, cont, command, max_retries):
                     f"az container exec -g {rg} -n {cg} --container-name {cont} --exec-command /bin/bash", "Enter",
                     ], env=env)
     time.sleep(5)
+    if input_file:
+        subprocess.Popen(["tmux", "send-keys", "-t", cont, f"python3 -m acido.cli -d {input_file}", "Enter"], env=env)
+        time.sleep(5)
     subprocess.Popen(["tmux", "send-keys", "-t", cont, f"nohup python3 -m acido.cli -sh '{command}' > temp &", "Enter"], env=env)
     time.sleep(2)
     subprocess.Popen(["tmux", "send-keys", "-t", cont, "Enter"], env=env)
@@ -102,21 +106,36 @@ def exec_command(rg, cg, cont, command, max_retries):
 
     retries = 0
     failed = False
+    exception = None
+    command_uuid = None
 
     while 'Done' not in output:
 
         retries += 1
 
         if retries > max_retries:
-            output = ''
+            exception = 'TIMEOUT REACHED'
             failed = True
             break
 
         subprocess.Popen(["tmux", "send-keys", "-t", cont, "Enter"], env=env)
-        time.sleep(5)
+        time.sleep(1)
         output = subprocess.check_output(["tmux", "capture-pane", "-pt", cont], env=env).decode()
 
         if 'Exit' in output:
+            subprocess.Popen(["tmux", "send-keys", "-t", cont, "Enter"], env=env)
+            time.sleep(2)
+            subprocess.Popen(["tmux", "send-keys", "-t", cont, "cat temp", "Enter", "Enter"], env=env)
+            time.sleep(2)
+            subprocess.Popen(["tmux", "send-keys", "-t", cont, "Enter"], env=env)
+            time.sleep(2)
+            try:
+                exception = subprocess.check_output(["tmux", "capture-pane", "-pt", cont], env=env)
+                exception = exception.decode()
+                exception = exception.split('cat temp')[1].strip()
+            except Exception as e:
+                exception = 'ERROR PARSING'
+                print(bad(f'Error capturing output from: {bold(cont)}'))
             failed = True
             break
         if 'Done' in output:
@@ -134,21 +153,19 @@ def exec_command(rg, cg, cont, command, max_retries):
             output = subprocess.check_output(["tmux", "capture-pane", "-pt", cont], env=env)
             output = output.decode()
             command_uuid = output.split('command: ')[1].split('\n')[0].strip()
-            print(command_uuid)
         except Exception as e:
             command_uuid = None
             print(bad(f'Error capturing output from: {bold(cont)}'))
     else:
-        command_uuid = None
         print(bad(f'Exception ocurred while executing "{command}" from: {bold(cont)}'))
 
     # Kill shell
-    subprocess.Popen(["tmux", "send-keys", "-t", cont, "(rm temp && rm output && exit)", "Enter"], env=env)
+    subprocess.Popen(["tmux", "send-keys", "-t", cont, "(rm temp && exit)", "Enter"], env=env)
     time.sleep(1)
     # Kill tmux window
     subprocess.Popen(["tmux", "kill-session", "-t", cont], env=env)
 
-    return cont, command_uuid
+    return cont, command_uuid, exception
 
 class Acido(object):
 
@@ -257,17 +274,15 @@ class Acido(object):
         if interactive:
             print(good(f"Listing all instances: [ {bold(' '.join(all_names))} ]"))
             print(good(f"Container group status: [ {' '.join([f'{bold(cg)}: {status}' for cg, status in all_instances_states.items()])} ]"))
-            return
-        return all_instances, all_instances_names
+        return None if interactive else all_instances, all_instances_names
     
     def select(self, selection, interactive=True):
         self.all_instances, self.instances_named = self.ls(interactive=False)
         selection = f'^{selection.replace("*", "(.*)")}$'
         self.selected_instances = [scg for scg, i in self.instances_named.items() if bool(re.match(selection, scg))]
         self._save_config()
-        if interactive:
-            print(good(f"Selected all instances of group/s: [ {bold(' '.join(self.selected_instances))} ]"))
-            return 
+        print(good(f"Selected all instances of group/s: [ {bold(' '.join(self.selected_instances))} ]"))
+        return None if interactive else self.selected_instances
 
     def fleet(self, fleet_name, instance_num=3, interactive=True):
         response = {}
@@ -308,20 +323,17 @@ class Acido(object):
             }
             response[fleet_name] = self.instance_manager.deploy(name=fleet_name, instance_number=instance_num, env_vars=env_vars)
         
-        if not interactive:
-            return response
-        else:
-            all_names = []
-            all_groups = []
+        all_names = []
+        all_groups = []
 
-            for group, inst in response.items():
-                all_groups.append(group)
-                all_names += list(inst.keys())
+        for group, inst in response.items():
+            all_groups.append(group)
+            all_names += list(inst.keys())
 
-            print(good(f"Successfully created new group/s: [ {bold(' '.join(all_groups))} ]"))
-            print(good(f"Successfully created new instance/s: [ {bold(' '.join(all_names))} ]"))
-
-            return
+        print(good(f"Successfully created new group/s: [ {bold(' '.join(all_groups))} ]"))
+        print(good(f"Successfully created new instance/s: [ {bold(' '.join(all_names))} ]"))
+        return None if interactive else response
+    
 
     def rm(self, selection):
         self.all_instances, self.instances_named = self.ls(interactive=False)
@@ -347,17 +359,25 @@ class Acido(object):
         if file:
             print(good(f'Executed command: {filename}'))
         return output
+
+    def save_input(self, filename: str = None):
+        file_contents = open(filename, 'rb').read()
+        file, filename = self.blob_manager.upload(
+            file_contents
+        )
+        if file:
+            print(good(f'Uploaded input: {filename}'))
+        return filename
     
-    def load_input(self, command_uuid: str = None, filename: str = 'input.txt', write_to_file: bool = False):
+    def load_input(self, command_uuid: str = None, filename: str = 'input', write_to_file: bool = False):
         if command_uuid:
             input_file = self.blob_manager.download(command_uuid)
             if write_to_file:
-                print(good(f'Downloaded file: {command_uuid}'))
                 open(filename, 'wb').write(input_file)
-                print(good(f'Input written to: {filename}'))
+                print(good(f'File loaded successfully.'))
         return input_file
 
-    def exec(self, command, max_retries=60, input_cmd: str = None):
+    def exec(self, command, max_retries=60, input_file: str = None):
         global instances_outputs
         self.all_instances, self.instances_named = self.ls(interactive=False)
         results = []
@@ -366,14 +386,48 @@ class Acido(object):
             print(bad('You didn\'t select any containers to execute the command.'))
             return
 
-        for cg, containers in self.instances_named.items():
-            if cg in self.selected_instances:
-                for cont in containers:
-                    executed = True
-                    result = pool.apply_async(exec_command, 
-                                             (self.rg, cg, cont, command, max_retries), 
-                                              callback=build_output)
-                    results.append(result)
+        if input_file:
+            number_of_containers = 0
+            for cg, containers in self.instances_named.items():
+                if cg in self.selected_instances:
+                    for cont in containers:
+                        number_of_containers += 1
+
+            number_of_lines = [line.strip() for line in open(input_file, 'r').readlines()]
+            chunked_lines = int(number_of_lines / number_of_containers)
+            os.system(f'split -l {str(chunked_lines)} {input_file} /tmp/acido-input-')
+        
+            directory = os.fsencode('/tmp/')
+            input_files = []
+            input_file_index = 0
+                
+            for file in os.listdir(directory):
+                filename = os.fsdecode(file)
+                if filename.startswith("acido-input"): 
+                    input_files.append(self.save_input(filename))
+                else:
+                    continue
+            
+            os.system('rm -f /tmp/acido-input*')
+            
+            for cg, containers in self.instances_named.items():
+                if cg in self.selected_instances:
+                    for cont in containers:
+                        executed = True
+                        result = pool.apply_async(exec_command, 
+                                                (self.rg, cg, cont, command, max_retries, input_files[input_file_index]), 
+                                                callback=build_output)
+                        results.append(result)
+                        input_file_index += 1
+        else:
+            for cg, containers in self.instances_named.items():
+                if cg in self.selected_instances:
+                    for cont in containers:
+                        executed = True
+                        result = pool.apply_async(exec_command, 
+                                                (self.rg, cg, cont, command, max_retries, input_file), 
+                                                callback=build_output)
+                        results.append(result)
         
         if not executed:
             print(bad('An error happened. You probably didn\'t select any containers to execute the command.'))
@@ -382,14 +436,15 @@ class Acido(object):
         results = [result.wait() for result in results]
 
         for c, o in instances_outputs.items():
-            o = self.load_input(o)
-            if o:
-                print(good(f'Executed command on {bold(c)}. Output: [\n{o}\n]'))
-            else:
+            command_uuid, exception = o
+            if command_uuid:
+                output = self.load_input(command_uuid)
+                print(good(f'Executed command on {bold(c)}. Output: [\n{output.decode().strip()}\n]'))
+            elif exception:
                 if max_retries == 0:
                     print(good(f'Executed command on {bold(c)}'))
                 else:
-                    print(bad(f'Executed command on {bold(c)} Output: FAILED TO PARSE'))
+                    print(bad(f'Executed command on {bold(c)} Output: [\n{exception}\n]'))
 
         instances_outputs = {}
 
@@ -411,11 +466,10 @@ if __name__ == "__main__":
     acido = Acido()
     if args.config:
         acido.setup()
-
     if args.shell:
         acido.save_output(args.shell)
     if args.download_input:
-        acido.load_input(args.download_input)
+        acido.load_input(args.download_input, write_to_file=True)
     if args.fleet:
         fleet_name = args.fleet if args.fleet else 'kali'
         instance_num = int(args.num_instances) if args.num_instances else 1
@@ -424,7 +478,7 @@ if __name__ == "__main__":
         acido.select(selection=args.select, interactive=bool(args.interactive))
     if args.exec_cmd:
         pool = ThreadPool(processes=30)
-        acido.exec(command=args.exec_cmd)
+        acido.exec(command=args.exec_cmd, max_retries=int(args.wait) if args.wait else 60, input_file=args.input_file)
     if args.remove:
         acido.rm(args.remove)
     if args.interactive:
